#!/bin/sh

# This script is used to determine if an updater is valid for the system upon
# which it is being run.
#
# COMMAND LINE
#
# One argument, a path to the file that needs testing
#
# RETURN CODE
#
# The return code is 0 if the script is successful, non-0 otherwise.
#
# OUTPUT
#
# The string 'true' is written to stdout, if the updater is valid.
# The string 'false: {reason}' is written to stdout, if the updater is not valid.
# The string 'error: {reason}' is written to stderr, if the script failed.
#
# NOTE: This code must be compatible with the initrd code that is used to
# validate updaters, see /usr/libexec/verifysystem in initrd-updater

set -u

source /lib/functions/navico_env.sh

# Modified from source/z/zzz_imx/overlay/initrd-updater/files/initrd/usr/libexec/updater/verifysystem
init_codename_regex() {
    local name=$CODENAME
    local arch=$PROCESSOR_FAMILY

    case "$arch" in
	mx6)    arch=":${arch}" ;;
	*)      arch=""         ;;
    esac

    local eol='\s*\(#.*\)*$'    # End of line regex
    local pfx='\([^ ,#]\+,\)*'  # Prefix regex
    local sfx='\(,[^ ,#]\+\)*'  # Suffix regex
    local sfid=$SUBFAMILYID

    CODENAME_RE="^CODENAME=${name}${arch}${eol}"
    CODENAME_RE="${CODENAME_RE}\|^CODENAMELIST=${pfx}${name}${arch}${sfx}${eol}"
    CODENAME_RE="${CODENAME_RE}\|^CODENAMELIST=${pfx}${name}-${sfid}${arch}${sfx}${eol}"
}

verify() {
    if [ -z "$1" ]; then
	echo "error: missing argument" 1>&2
	return 1

    elif [ ! -r "$1" ]; then
	echo "error: unreadable file: '$1'" 1>&2
	return 1

    else
	navico_env_init

	if [ -z "$CODENAME" ]; then
	    echo 'error: missing host codename' 1>&2
	    return 1
	fi

	init_codename_regex

	local upd_codenames

	if ! upd_codenames=$(tar -xOf "$1" update.sh 2>/dev/null | grep '^CODENAME'); then
	    echo 'error: not an updater' 1>&2
	    return 1
	fi

	if printf "%s\n" "$upd_codenames" | grep -q "$CODENAME_RE"; then
	    echo 'true'
	else
	    echo 'false: codename mismatch'
	fi
    fi
}

verify "${1:-}"
exit $?
